@@ -11,7 +11,10 @@ from qtpy.QtWidgets import (
 
 import napari
 
+from ...utils.action_manager import action_manager
 from ...utils.interactions import get_key_bindings_summary
+from ...utils.theme import get_theme
+from ...utils.translations import trans
 
 
 class QtAboutKeyBindings(QDialog):
@@ -22,6 +25,9 @@ class QtAboutKeyBindings(QDialog):
     viewer : napari.components.ViewerModel
         Napari viewer containing the rendered scene, layers, and controls.
 
+    key_map_handler : napari.utils.key_bindings.KeyMapHandler
+        Handler for key mapping and calling functionality.
+
     Attributes
     ----------
     key_bindings_strs : collections.OrderedDict
@@ -44,15 +50,15 @@ class QtAboutKeyBindings(QDialog):
         Napari viewer containing the rendered scene, layers, and controls.
     """
 
-    ALL_ACTIVE_KEYBINDINGS = 'All active key bindings'
+    ALL_ACTIVE_KEYBINDINGS = trans._('All active key bindings')
 
-    def __init__(self, viewer, parent=None):
+    def __init__(self, viewer, key_map_handler, parent=None):
         super().__init__(parent=parent)
 
         self.viewer = viewer
         self.layout = QVBoxLayout()
 
-        self.setWindowTitle('Keybindings')
+        self.setWindowTitle(trans._('Keybindings'))
         self.setWindowModality(Qt.NonModal)
         self.setLayout(self.layout)
 
@@ -60,10 +66,14 @@ class QtAboutKeyBindings(QDialog):
         self.textEditBox = QTextEdit()
         self.textEditBox.setTextInteractionFlags(Qt.TextSelectableByMouse)
         self.textEditBox.setMinimumWidth(360)
+
         # Can switch to a normal dict when our minimum Python is 3.7
         self.key_bindings_strs = OrderedDict()
         self.key_bindings_strs[self.ALL_ACTIVE_KEYBINDINGS] = ''
-        col = self.viewer.palette['secondary']
+        self.key_map_handler = key_map_handler
+        theme = get_theme(self.viewer.theme)
+
+        col = theme['secondary']
         layers = [
             napari.layers.Image,
             napari.layers.Labels,
@@ -72,19 +82,24 @@ class QtAboutKeyBindings(QDialog):
             napari.layers.Surface,
             napari.layers.Vectors,
         ]
+        layer_shortcuts = action_manager._get_layer_shortcuts(layers)
         for layer in layers:
             if len(layer.class_keymap) == 0:
-                text = 'No key bindings'
+                text = trans._('No key bindings')
             else:
-                text = get_key_bindings_summary(layer.class_keymap, col=col)
+                text = get_key_bindings_summary(
+                    layer_shortcuts[layer], col=col
+                )
+
+            # TODO: Add localization. Add localized layer name to layer types.
             self.key_bindings_strs[f"{layer.__name__} layer"] = text
 
         # layer type selection
         self.layerTypeComboBox = QComboBox()
         self.layerTypeComboBox.addItems(list(self.key_bindings_strs))
+
         self.layerTypeComboBox.activated[str].connect(self.change_layer_type)
         self.layerTypeComboBox.setCurrentText(self.ALL_ACTIVE_KEYBINDINGS)
-        # self.change_layer_type(current_layer)
         layer_type_layout = QHBoxLayout()
         layer_type_layout.setContentsMargins(10, 5, 0, 0)
         layer_type_layout.addWidget(self.layerTypeComboBox)
@@ -93,8 +108,7 @@ class QtAboutKeyBindings(QDialog):
         self.layout.addLayout(layer_type_layout)
         self.layout.addWidget(self.textEditBox, 1)
 
-        self.viewer.events.active_layer.connect(self.update_active_layer)
-        self.viewer.events.palette.connect(self.update_active_layer)
+        self.viewer.events.theme.connect(self.update_active_layer)
         self.update_active_layer()
 
     def change_layer_type(self, text):
@@ -123,9 +137,14 @@ class QtAboutKeyBindings(QDialog):
         event : napari.utils.event.Event, optional
             The napari event that triggered this method, by default None.
         """
-        col = self.viewer.palette['secondary']
+        theme = get_theme(self.viewer.theme)
+        col = theme['secondary']
+
         # Add class and instance viewer key bindings
-        text = get_key_bindings_summary(self.viewer.active_keymap, col=col)
+        active_shortcuts = action_manager._get_active_shortcuts(
+            self.key_map_handler.active_keymap
+        )
+        text = get_key_bindings_summary(active_shortcuts, col=col)
 
         # Update layer speficic key bindings if all active are displayed
         self.key_bindings_strs[self.ALL_ACTIVE_KEYBINDINGS] = text
