@@ -11,7 +11,6 @@ from vispy.color import get_colormap
 from vispy.visuals.shaders import Function, FunctionChain
 from vispy.visuals.transforms import NullTransform
 from vispy.visuals.visual import Visual
-from vispy.ext.six import string_types
 from vispy.io import load_spatial_filters
 
 VERT_SHADER = """
@@ -113,8 +112,8 @@ _null_color_transform = 'vec4 pass(vec4 color) { return color; }'
 _c2l = 'float cmap(vec4 color) { return (color.r + color.g + color.b) / 3.; }'
 
 
-def _build_color_transform(data, clim, gamma, cmap):
-    if data.ndim == 2 or data.shape[2] == 1:
+def _build_color_transform(grayscale: bool, clim, gamma, cmap):
+    if grayscale:
         fclim = Function(_apply_clim_float)
         fgamma = Function(_apply_gamma_float)
         fun = FunctionChain(
@@ -306,15 +305,11 @@ class ImageVisual(Visual):
 
     @property
     def clim(self):
-        return (
-            self._clim
-            if isinstance(self._clim, string_types)
-            else tuple(self._clim)
-        )
+        return self._clim if isinstance(self._clim, str) else tuple(self._clim)
 
     @clim.setter
     def clim(self, clim):
-        if isinstance(clim, string_types):
+        if isinstance(clim, str):
             if clim != 'auto':
                 raise ValueError('clim must be "auto" if a string')
             self._need_texture_upload = True
@@ -498,7 +493,7 @@ class ImageVisual(Visual):
             # deal with clim on CPU b/c of texture depth limits :(
             # can eventually do this by simulating 32-bit float... maybe
             clim = self._clim
-            if isinstance(clim, string_types) and clim == 'auto':
+            if isinstance(clim, str) and clim == 'auto':
                 clim = np.min(data), np.max(data)
             clim = np.asarray(clim, dtype=np.float32)
             data = data - clim[0]  # not inplace so we don't modify orig data
@@ -509,10 +504,11 @@ class ImageVisual(Visual):
             self._clim = np.array(clim)
         else:
             # assume that RGB data is already scaled (0, 1)
-            if isinstance(self._clim, string_types) and self._clim == 'auto':
+            if isinstance(self._clim, str) and self._clim == 'auto':
                 self._clim = (0, 1)
 
         self._texture_limits = np.array(self._clim)
+        self._need_colortransform_update = True
         self._texture.set_data(data)
         self._need_texture_upload = False
 
@@ -545,10 +541,11 @@ class ImageVisual(Visual):
 
         if self._need_colortransform_update:
             prg = view.view_program
+            grayscale = self._data.ndim == 2 or self._data.shape[2] == 1
             self.shared_program.frag[
                 'color_transform'
             ] = _build_color_transform(
-                self._data, self.clim_normalized, self.gamma, self.cmap
+                grayscale, self.clim_normalized, self.gamma, self.cmap
             )
             self._need_colortransform_update = False
             prg['texture2D_LUT'] = (
