@@ -1,12 +1,9 @@
-import warnings
+import os
 from functools import partial
-from typing import List
 
 import numpy as np
 import pytest
-from qtpy.QtWidgets import QApplication
 
-from napari import Viewer
 from napari.components import LayerList
 from napari.layers import Image, Labels, Points, Shapes, Vectors
 from napari.plugins._builtins import (
@@ -16,12 +13,12 @@ from napari.plugins._builtins import (
     napari_write_shapes,
 )
 from napari.utils import io
+from napari.utils.config import async_loading
 
 try:
     from skimage.data import image_fetcher
 except ImportError:
     from skimage.data import data_dir
-    import os
 
     class image_fetcher:
         def fetch(data_name):
@@ -36,57 +33,25 @@ except ImportError:
 
 
 def pytest_addoption(parser):
-    """An option to show viewers during tests. (Hidden by default).
+    """Add napari specific command line options.
 
-    Showing viewers decreases test speed by about %18.  Note, due to the
-    placement of this conftest.py file, you must specify the napari folder (in
-    the pytest command) to use this flag.
+    --aysnc_only
+        Run only asynchronous tests, not sync ones.
 
-    Examples
-    --------
-    $ pytest napari --show-viewer
+    Notes
+    -----
+    Due to the placement of this conftest.py file, you must specifically name
+    the napari folder such as "pytest napari --aysnc_only"
     """
+
     parser.addoption(
-        "--show-viewer",
+        "--async_only",
         action="store_true",
         default=False,
-        help="don't show viewer during tests",
+        help="run only asynchronous tests",
     )
 
 
-@pytest.fixture
-def qtbot(qtbot):
-    """A modified qtbot fixture that makes sure no widgets have been leaked."""
-    initial = QApplication.topLevelWidgets()
-    yield qtbot
-    QApplication.processEvents()
-    leaks = set(QApplication.topLevelWidgets()).difference(initial)
-    # still not sure how to clean up some of the remaining vispy
-    # vispy.app.backends._qt.CanvasBackendDesktop widgets...
-    if any([n.__class__.__name__ != 'CanvasBackendDesktop' for n in leaks]):
-        raise AssertionError(f'Widgets leaked!: {leaks}')
-    if leaks:
-        warnings.warn(f'Widgets leaked!: {leaks}')
-
-
-@pytest.fixture(scope="function")
-def make_test_viewer(qtbot, request):
-    viewers: List[Viewer] = []
-
-    def actual_factory(*model_args, **model_kwargs):
-        model_kwargs['show'] = model_kwargs.pop(
-            'show', request.config.getoption("--show-viewer")
-        )
-        viewer = Viewer(*model_args, **model_kwargs)
-        viewers.append(viewer)
-        return viewer
-
-    yield actual_factory
-
-    for viewer in viewers:
-        viewer.close()
-
-
 @pytest.fixture(
     params=['image', 'labels', 'points', 'points-with-properties', 'shapes']
 )
@@ -189,9 +154,9 @@ def layer_data_and_types():
         ),
     ]
     extensions = ['.tif', '.tif', '.csv', '.csv']
-    layer_data = [l.as_layer_data_tuple() for l in layers]
+    layer_data = [layer.as_layer_data_tuple() for layer in layers]
     layer_types = [layer._type_string for layer in layers]
-    filenames = [l.name + e for l, e in zip(layers, extensions)]
+    filenames = [layer.name + e for layer, e in zip(layers, extensions)]
     return layers, layer_data, layer_types, filenames
 
 
@@ -291,3 +256,79 @@ def irregular_images():
 @pytest.fixture
 def single_tiff():
     return [image_fetcher.fetch('data/multipage.tif')]
+
+
+# Currently we cannot run async and async in the invocation of pytest
+# because we get a segfault for unknown reasons. So for now:
+# "pytest" runs sync_only
+# "pytest napari --async_only" runs async only
+@pytest.fixture(scope="session", autouse=True)
+def configure_loading(request):
+    """Configure async/async loading."""
+    async_mode = request.config.getoption("--async_only")
+    if async_mode:
+        # Late import so we don't import experimental code unless using it.
+        from napari.components.experimental.chunk import synchronous_loading
+
+        with synchronous_loading(False):
+            yield
+    else:
+        yield  # Sync so do nothing.
+
+
+def _is_async_mode() -> bool:
+    """Return True if we are currently loading chunks asynchronously
+
+    Returns
+    -------
+    bool
+        True if we are currently loading chunks asynchronously.
+    """
+    if not async_loading:
+        return False  # Not enabled at all.
+    else:
+        # Late import so we don't import experimental code unless using it.
+        from napari.components.experimental.chunk import chunk_loader
+
+        return not chunk_loader.force_synchronous
+
+
+@pytest.fixture(autouse=True)
+def skip_sync_only(request):
+    """Skip async_only tests if running async."""
+    sync_only = request.node.get_closest_marker('sync_only')
+    if _is_async_mode() and sync_only:
+        pytest.skip("running with --async_only")
+
+
+@pytest.fixture(autouse=True)
+def skip_async_only(request):
+    """Skip async_only tests if running sync."""
+    async_only = request.node.get_closest_marker('async_only')
+    if not _is_async_mode() and async_only:
+        pytest.skip("not running with --async_only")
+
+
+# _PYTEST_RAISE=1 will prevent pytest from handling exceptions.
+# Use with a debugger that's set to break on "unhandled exceptions".
+# https://github.com/pytest-dev/pytest/issues/7409
+if os.getenv('_PYTEST_RAISE', "0") != "0":
+
+    @pytest.hookimpl(tryfirst=True)
+    def pytest_exception_interact(call):
+        raise call.excinfo.value
+
+    @pytest.hookimpl(tryfirst=True)
+    def pytest_internalerror(excinfo):
+        raise excinfo.value
+
+
+def pytest_collection_modifyitems(session, config, items):
+
+    put_at_end = ('test_trace_on_start',)
+    at_end = []
+    for i, item in enumerate(items):
+        if item.name in put_at_end:
+            at_end.append(items.pop(i))
+
+    items.extend([x for _, x in sorted(zip(put_at_end, at_end))])
