@@ -1,10 +1,12 @@
 from copy import copy
 from itertools import cycle, islice
+
 import numpy as np
 import pandas as pd
 import pytest
 from vispy.color import get_colormap
 
+from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Points
 from napari.layers.points._points_utils import points_to_squares
 from napari.utils.colormaps.standardize_color import transform_color
@@ -35,39 +37,39 @@ def test_empty_points():
 
 
 def test_empty_points_with_properties():
-    """ Test instantiating an empty Points layer with properties
+    """Test instantiating an empty Points layer with properties
 
     See: https://github.com/napari/napari/pull/1069
     """
     properties = {
         'label': np.array(['label1', 'label2']),
-        'cont_prop': np.array([0], dtype=np.float),
+        'cont_prop': np.array([0], dtype=float),
     }
-    pts = Points(properties=properties)
+    pts = Points(property_choices=properties)
     current_props = {k: v[0] for k, v in properties.items()}
     np.testing.assert_equal(pts.current_properties, current_props)
 
     # verify the property datatype is correct
-    assert pts.properties['cont_prop'].dtype == np.float
+    assert pts.properties['cont_prop'].dtype == float
 
     # add two points and verify the default property was applied
     pts.add([10, 10])
     pts.add([20, 20])
     props = {
         'label': np.array(['label1', 'label1']),
-        'cont_prop': np.array([0, 0], dtype=np.float),
+        'cont_prop': np.array([0, 0], dtype=float),
     }
     np.testing.assert_equal(pts.properties, props)
 
 
 def test_empty_points_with_properties_list():
-    """ Test instantiating an empty Points layer with properties
+    """Test instantiating an empty Points layer with properties
     stored in a list
 
     See: https://github.com/napari/napari/pull/1069
     """
     properties = {'label': ['label1', 'label2'], 'cont_prop': [0]}
-    pts = Points(properties=properties)
+    pts = Points(property_choices=properties)
     current_props = {k: np.asarray(v[0]) for k, v in properties.items()}
     np.testing.assert_equal(pts.current_properties, current_props)
 
@@ -76,45 +78,76 @@ def test_empty_points_with_properties_list():
     pts.add([20, 20])
     props = {
         'label': np.array(['label1', 'label1']),
-        'cont_prop': np.array([0, 0], dtype=np.float),
+        'cont_prop': np.array([0, 0], dtype=float),
     }
     np.testing.assert_equal(pts.properties, props)
 
 
-def test_empty_layer_with_face_colorap():
-    """ Test creating an empty layer where the face color is a colormap
+def test_empty_layer_with_face_colormap():
+    """Test creating an empty layer where the face color is a colormap
     See: https://github.com/napari/napari/pull/1069
     """
-    default_properties = {'point_type': np.array([1.5], dtype=np.float)}
+    default_properties = {'point_type': np.array([1.5], dtype=float)}
     layer = Points(
-        properties=default_properties,
+        property_choices=default_properties,
         face_color='point_type',
-        face_colormap='grays',
+        face_colormap='gray',
     )
 
     assert layer.face_color_mode == 'colormap'
 
     # verify the current_face_color is correct
     face_color = np.array([1, 1, 1, 1])
-    assert np.all(layer._current_face_color == face_color)
+    np.testing.assert_allclose(layer._face.current_color, face_color)
 
 
 def test_empty_layer_with_edge_colormap():
-    """ Test creating an empty layer where the face color is a colormap
+    """Test creating an empty layer where the face color is a colormap
     See: https://github.com/napari/napari/pull/1069
     """
-    default_properties = {'point_type': np.array([1.5], dtype=np.float)}
+    default_properties = {'point_type': np.array([1.5], dtype=float)}
     layer = Points(
-        properties=default_properties,
+        property_choices=default_properties,
         edge_color='point_type',
-        edge_colormap='grays',
+        edge_colormap='gray',
     )
 
     assert layer.edge_color_mode == 'colormap'
 
     # verify the current_face_color is correct
     edge_color = np.array([1, 1, 1, 1])
-    assert np.all(layer._current_edge_color == edge_color)
+    np.testing.assert_allclose(layer._edge.current_color, edge_color)
+
+
+def test_empty_layer_with_text_properties():
+    """Test initializing an empty layer with text defined"""
+    default_properties = {'point_type': np.array([1.5], dtype=float)}
+    text_kwargs = {'text': 'point_type', 'color': 'red'}
+    layer = Points(
+        property_choices=default_properties,
+        text=text_kwargs,
+    )
+    np.testing.assert_equal(layer.text.values, np.empty(0))
+    np.testing.assert_allclose(layer.text.color, [1, 0, 0, 1])
+
+    # add a point and check that the appropriate text value was added
+    layer.add([1, 1])
+    np.testing.assert_equal(layer.text.values, ['1.5'])
+    np.testing.assert_allclose(layer.text.color, [1, 0, 0, 1])
+
+
+def test_empty_layer_with_text_formatted():
+    """Test initializing an empty layer with text defined"""
+    default_properties = {'point_type': np.array([1.5], dtype=float)}
+    layer = Points(
+        property_choices=default_properties,
+        text='point_type: {point_type:.2f}',
+    )
+    np.testing.assert_equal(layer.text.values, np.empty(0))
+
+    # add a point and check that the appropriate text value was added
+    layer.add([1, 1])
+    np.testing.assert_equal(layer.text.values, ['point_type: 1.50'])
 
 
 def test_random_points():
@@ -216,7 +249,7 @@ def test_selecting_points():
     assert layer.selected_data == data_to_select
 
     # test switching to 3D
-    layer.dims.ndisplay = 3
+    layer._slice_dims(ndisplay=3)
     assert layer.selected_data == data_to_select
 
     # select different points while in 3D mode
@@ -225,7 +258,7 @@ def test_selecting_points():
     assert layer.selected_data == other_data_to_select
 
     # selection should persist when going back to 2D mode
-    layer.dims.ndisplay = 2
+    layer._slice_dims(ndisplay=2)
     assert layer.selected_data == other_data_to_select
 
     # selection should persist when switching between between select and pan_zoom
@@ -335,7 +368,6 @@ def test_changing_modes():
 
     layer.mode = 'add'
     assert layer.mode == 'add'
-    assert layer.interactive is False
 
     layer.mode = 'select'
     assert layer.mode == 'select'
@@ -498,9 +530,14 @@ def test_adding_properties(attribute):
     assert isinstance(layer.properties['point_type'], np.ndarray)
 
     # removing a property that was the _*_color_property should give a warning
-    setattr(layer, f'_{attribute}_color_property', 'vector_type')
+    color_manager = getattr(layer, f'_{attribute}')
+    color_manager.color_properties = {
+        'name': 'point_type',
+        'values': np.empty(0),
+        'current_value': 'A',
+    }
     properties_2 = {
-        'not_vector_type': _make_cycled_properties(['A', 'B'], shape[0])
+        'not_point_type': _make_cycled_properties(['A', 'B'], shape[0])
     }
     with pytest.warns(RuntimeWarning):
         layer.properties = properties_2
@@ -660,27 +697,6 @@ def test_points_errors():
         Points(data, properties=copy(annotations))
 
 
-def test_is_color_mapped():
-    shape = (10, 2)
-    np.random.seed(0)
-    data = 20 * np.random.random(shape)
-    annotations = {'point_type': _make_cycled_properties(['A', 'B'], shape[0])}
-    layer = Points(data, properties=annotations)
-
-    # giving the name of an annotation should return True
-    assert layer._is_color_mapped('point_type')
-
-    # giving a list should return false (i.e., could be an RGBA color)
-    assert not layer._is_color_mapped([1, 1, 1, 1])
-
-    # giving an ndarray should return false (i.e., could be an RGBA color)
-    assert not layer._is_color_mapped(np.array([1, 1, 1, 1]))
-
-    # give an invalid color argument
-    with pytest.raises(ValueError):
-        layer._is_color_mapped((123, 323))
-
-
 def test_edge_width():
     """Test setting edge width."""
     shape = (10, 2)
@@ -722,6 +738,7 @@ def test_n_dimensional():
     assert layer.n_dimensional is True
 
 
+@pytest.mark.filterwarnings("ignore:elementwise comparison fail:FutureWarning")
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 def test_switch_color_mode(attribute):
     """Test switching between color modes"""
@@ -755,16 +772,18 @@ def test_switch_color_mode(attribute):
     )
 
     # there should not be an edge_color_property
-    color_property = getattr(layer, f'_{attribute}_color_property')
-    assert color_property == ''
+    color_manager = getattr(layer, f'_{attribute}')
+    color_property = color_manager.color_properties
+    assert color_property is None
 
     # transitioning to colormap should raise a warning
     # because there isn't an edge color property yet and
     # the first property in points.properties is being automatically selected
     with pytest.warns(UserWarning):
         setattr(layer, f'{attribute}_color_mode', 'colormap')
-    color_property = getattr(layer, f'_{attribute}_color_property')
-    assert color_property == next(iter(properties))
+    color_manager = getattr(layer, f'_{attribute}')
+    color_property_name = color_manager.color_properties.name
+    assert color_property_name == next(iter(properties))
     layer_color = getattr(layer, f'{attribute}_color')
     np.testing.assert_allclose(layer_color[-1], [1, 1, 1, 1])
 
@@ -772,7 +791,7 @@ def test_switch_color_mode(attribute):
     setattr(layer, f'{attribute}_color_mode', 'cycle')
     setattr(layer, f'{attribute}_color', 'point_type')
     color = getattr(layer, f'{attribute}_color')
-    layer_color = transform_color(color_cycle * int((shape[0] / 2)))
+    layer_color = transform_color(color_cycle * int(shape[0] / 2))
     np.testing.assert_allclose(color, layer_color)
 
     # switch back to direct, edge_colors shouldn't change
@@ -803,7 +822,8 @@ def test_colormap_with_categorical_properties(attribute):
     layer = Points(data, properties=properties)
 
     with pytest.raises(TypeError):
-        setattr(layer, f'{attribute}_color_mode', 'colormap')
+        with pytest.warns(UserWarning):
+            setattr(layer, f'{attribute}_color_mode', 'colormap')
 
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
@@ -820,7 +840,20 @@ def test_add_colormap(attribute):
 
     setattr(layer, f'{attribute}_colormap', get_colormap('gray'))
     layer_colormap = getattr(layer, f'{attribute}_colormap')
-    assert 'unnamed colormap' in layer_colormap[0]
+    assert 'unnamed colormap' in layer_colormap.name
+
+
+@pytest.mark.parametrize("attribute", ['edge', 'face'])
+def test_add_point_direct(attribute: str):
+    """Test adding points to layer directly"""
+    layer = Points()
+    assert len(getattr(layer, f'{attribute}_color')) == 0
+    setattr(layer, f'current_{attribute}_color', 'red')
+    coord = [18, 18]
+    layer.add(coord)
+    np.testing.assert_allclose(
+        [[1, 0, 0, 1]], getattr(layer, f'{attribute}_color')
+    )
 
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
@@ -873,7 +906,8 @@ def test_color_direct(attribute: str):
     layer_color = getattr(layer, f'{attribute}_color')
     assert len(layer_color) == shape[0] - 1
     np.testing.assert_allclose(
-        layer_color, np.vstack((color_array[1], color_array[3:])),
+        layer_color,
+        np.vstack((color_array[1], color_array[3:])),
     )
 
 
@@ -884,7 +918,8 @@ color_cycle_rgba = [[1, 0, 0, 1], [0, 0, 1, 1]]
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 @pytest.mark.parametrize(
-    "color_cycle", [color_cycle_str, color_cycle_rgb, color_cycle_rgba],
+    "color_cycle",
+    [color_cycle_str, color_cycle_rgb, color_cycle_rgba],
 )
 def test_color_cycle(attribute, color_cycle):
     """Test setting edge/face color with a color cycle list"""
@@ -914,7 +949,8 @@ def test_color_cycle(attribute, color_cycle):
     layer_color = getattr(layer, f'{attribute}_color')
     assert len(layer_color) == shape[0] + 1
     np.testing.assert_allclose(
-        layer_color, np.vstack((color_array, transform_color('red'))),
+        layer_color,
+        np.vstack((color_array, transform_color('red'))),
     )
 
     # Check removing data adjusts colors correctly
@@ -929,16 +965,14 @@ def test_color_cycle(attribute, color_cycle):
         np.vstack((color_array[1], color_array[3:], transform_color('red'))),
     )
 
-    # refresh colors
-    layer.refresh_colors(update_color_mapping=True)
-
     # test adding a point with a new property value
     layer.selected_data = {}
     current_properties = layer.current_properties
     current_properties['point_type'] = np.array(['new'])
     layer.current_properties = current_properties
     layer.add([10, 10])
-    color_cycle_map = getattr(layer, f'{attribute}_color_cycle_map')
+    color_manager = getattr(layer, f'_{attribute}')
+    color_cycle_map = color_manager.categorical_colormap.colormap
 
     assert 'new' in color_cycle_map
     np.testing.assert_allclose(
@@ -946,24 +980,44 @@ def test_color_cycle(attribute, color_cycle):
     )
 
 
+@pytest.mark.parametrize("attribute", ['edge', 'face'])
+def test_color_cycle_dict(attribute):
+    """Test setting edge/face color with a color cycle dict"""
+    data = np.array([[0, 0], [100, 0], [0, 100]])
+    properties = {'my_colors': [2, 6, 3]}
+    points_kwargs = {
+        'properties': properties,
+        f'{attribute}_color': 'my_colors',
+        f'{attribute}_color_cycle': {1: 'green', 2: 'red', 3: 'blue'},
+    }
+    layer = Points(data, **points_kwargs)
+
+    color_manager = getattr(layer, f'_{attribute}')
+    color_cycle_map = color_manager.categorical_colormap.colormap
+    np.testing.assert_allclose(color_cycle_map[2], [1, 0, 0, 1])  # 2 is red
+    np.testing.assert_allclose(color_cycle_map[3], [0, 0, 1, 1])  # 3 is blue
+    np.testing.assert_allclose(color_cycle_map[6], [1, 1, 1, 1])  # 6 is white
+
+
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 def test_add_color_cycle_to_empty_layer(attribute):
-    """ Test adding a point to an empty layer when edge/face color is a color cycle
+    """Test adding a point to an empty layer when edge/face color is a color cycle
 
     See: https://github.com/napari/napari/pull/1069
     """
     default_properties = {'point_type': np.array(['A'])}
     color_cycle = ['red', 'blue']
     points_kwargs = {
-        'properties': default_properties,
+        'property_choices': default_properties,
         f'{attribute}_color': 'point_type',
         f'{attribute}_color_cycle': color_cycle,
     }
     layer = Points(**points_kwargs)
 
     # verify the current_edge_color is correct
-    expected_color = transform_color(color_cycle[0])
-    current_color = getattr(layer, f'_current_{attribute}_color')
+    expected_color = transform_color(color_cycle[0])[0]
+    color_manager = getattr(layer, f'_{attribute}')
+    current_color = color_manager.current_color
     np.testing.assert_allclose(current_color, expected_color)
 
     # add a point
@@ -988,7 +1042,7 @@ def test_add_color_cycle_to_empty_layer(attribute):
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 def test_adding_value_color_cycle(attribute):
-    """ Test that adding values to properties used to set a color cycle
+    """Test that adding values to properties used to set a color cycle
     and then calling Points.refresh_colors() performs the update and adds the
     new value to the face/edge_color_cycle_map.
 
@@ -1007,12 +1061,14 @@ def test_adding_value_color_cycle(attribute):
     layer = Points(data, **points_kwargs)
 
     # make point 0 point_type C
-    point_types = layer.properties['point_type']
+    props = layer.properties
+    point_types = props['point_type']
     point_types[0] = 'C'
-    layer.properties['point_type'] = point_types
-    layer.refresh_colors(update_color_mapping=False)
+    props['point_type'] = point_types
+    layer.properties = props
 
-    color_cycle_map = getattr(layer, f'{attribute}_color_cycle_map')
+    color_manager = getattr(layer, f'_{attribute}')
+    color_cycle_map = color_manager.categorical_colormap.colormap
     color_map_keys = [*color_cycle_map]
     assert 'C' in color_map_keys
 
@@ -1034,7 +1090,7 @@ def test_color_colormap(attribute):
     assert layer.properties == properties
     color_mode = getattr(layer, f'{attribute}_color_mode')
     assert color_mode == 'colormap'
-    color_array = transform_color(['black', 'white'] * int((shape[0] / 2)))
+    color_array = transform_color(['black', 'white'] * int(shape[0] / 2))
     attribute_color = getattr(layer, f'{attribute}_color')
     assert np.all(attribute_color == color_array)
 
@@ -1050,7 +1106,8 @@ def test_color_colormap(attribute):
     attribute_color = getattr(layer, f'{attribute}_color')
     assert len(attribute_color) == shape[0] + 1
     np.testing.assert_allclose(
-        attribute_color, np.vstack((color_array, transform_color('black'))),
+        attribute_color,
+        np.vstack((color_array, transform_color('black'))),
     )
 
     # Check removing data adjusts colors correctly
@@ -1062,13 +1119,16 @@ def test_color_colormap(attribute):
     np.testing.assert_allclose(
         attribute_color,
         np.vstack(
-            (color_array[1], color_array[3:], transform_color('black'),)
+            (
+                color_array[1],
+                color_array[3:],
+                transform_color('black'),
+            )
         ),
     )
 
     # adjust the clims
     setattr(layer, f'{attribute}_contrast_limits', (0, 3))
-    layer.refresh_colors(update_color_mapping=False)
     attribute_color = getattr(layer, f'{attribute}_color')
     np.testing.assert_allclose(attribute_color[-2], [0.5, 0.5, 0.5, 1])
 
@@ -1076,7 +1136,7 @@ def test_color_colormap(attribute):
     new_colormap = 'viridis'
     setattr(layer, f'{attribute}_colormap', new_colormap)
     attribute_colormap = getattr(layer, f'{attribute}_colormap')
-    assert attribute_colormap[1] == get_colormap(new_colormap)
+    assert attribute_colormap.name == new_colormap
 
 
 def test_size():
@@ -1304,12 +1364,11 @@ def test_value():
     data = 20 * np.random.random(shape)
     data[-1] = [0, 0]
     layer = Points(data)
-    value = layer.get_value()
-    assert layer.coordinates == (0, 0)
+    value = layer.get_value((0, 0))
     assert value == 9
 
     layer.data = layer.data + 20
-    value = layer.get_value()
+    value = layer.get_value((0, 0))
     assert value is None
 
 
@@ -1320,7 +1379,7 @@ def test_message():
     data = 20 * np.random.random(shape)
     data[-1] = [0, 0]
     layer = Points(data)
-    msg = layer.get_message()
+    msg = layer.get_status((0,) * 2)
     assert type(msg) == str
 
 
@@ -1351,7 +1410,7 @@ def test_thumbnail_with_n_points_greater_than_max():
     # #3D
     bigger_data_3d = np.random.randint(10, 100, (max_points, 3))
     bigger_layer_3d = Points(bigger_data_3d)
-    bigger_layer_3d.dims.ndisplay = 3
+    bigger_layer_3d._slice_dims(ndisplay=3)
     bigger_layer_3d._update_thumbnail()
     assert bigger_layer_3d.thumbnail.shape == bigger_layer_3d._thumbnail_shape
 
@@ -1360,17 +1419,17 @@ def test_view_data():
     coords = np.array([[0, 1, 1], [0, 2, 2], [1, 3, 3], [3, 3, 3]])
     layer = Points(coords)
 
-    layer.dims.set_point(0, 0)
+    layer._slice_dims([0, slice(None), slice(None)])
     assert np.all(
-        layer._view_data == coords[np.ix_([0, 1], layer.dims.displayed)]
+        layer._view_data == coords[np.ix_([0, 1], layer._dims_displayed)]
     )
 
-    layer.dims.set_point(0, 1)
+    layer._slice_dims([1, slice(None), slice(None)])
     assert np.all(
-        layer._view_data == coords[np.ix_([2], layer.dims.displayed)]
+        layer._view_data == coords[np.ix_([2], layer._dims_displayed)]
     )
 
-    layer.dims.ndisplay = 3
+    layer._slice_dims([1, slice(None), slice(None)], ndisplay=3)
     assert np.all(layer._view_data == coords)
 
 
@@ -1379,20 +1438,22 @@ def test_view_size():
     sizes = np.array([[3, 5, 5], [3, 5, 5], [3, 3, 3], [2, 2, 3]])
     layer = Points(coords, size=sizes, n_dimensional=False)
 
-    layer.dims.set_point(0, 0)
+    layer._slice_dims([0, slice(None), slice(None)])
     assert np.all(
-        layer._view_size == sizes[np.ix_([0, 1], layer.dims.displayed)]
+        layer._view_size == sizes[np.ix_([0, 1], layer._dims_displayed)]
     )
 
-    layer.dims.set_point(0, 1)
-    assert np.all(layer._view_size == sizes[np.ix_([2], layer.dims.displayed)])
+    layer._slice_dims([1, slice(None), slice(None)])
+    assert np.all(
+        layer._view_size == sizes[np.ix_([2], layer._dims_displayed)]
+    )
 
     layer.n_dimensional = True
     assert len(layer._view_size) == 3
 
     # test a slice with no points
     layer.n_dimensional = False
-    layer.dims.set_point(0, 2)
+    layer._slice_dims([2, slice(None), slice(None)])
     assert np.all(layer._view_size == [])
 
 
@@ -1406,18 +1467,16 @@ def test_view_colors():
     )
 
     layer = Points(coords, face_color=face_color, edge_color=edge_color)
-    layer.dims.set_point(0, 0)
-    print(layer.face_color)
-    print(layer._view_face_color)
+    layer._slice_dims([0, slice(None), slice(None)])
     assert np.all(layer._view_face_color == face_color[[0, 1]])
     assert np.all(layer._view_edge_color == edge_color[[0, 1]])
 
-    layer.dims.set_point(0, 1)
+    layer._slice_dims([1, slice(None), slice(None)])
     assert np.all(layer._view_face_color == face_color[[2]])
     assert np.all(layer._view_edge_color == edge_color[[2]])
 
     # view colors should return empty array if there are no points
-    layer.dims.set_point(0, 2)
+    layer._slice_dims([2, slice(None), slice(None)])
     assert len(layer._view_face_color) == 0
     assert len(layer._view_edge_color) == 0
 
@@ -1438,3 +1497,99 @@ def test_interaction_box():
     expected_box = points_to_squares(data, size)
     box = layer.interaction_box(index)
     np.all([np.isin(p, expected_box) for p in box])
+
+
+def test_world_data_extent():
+    """Test extent after applying transforms."""
+    data = [(7, -5, 0), (-2, 0, 15), (4, 30, 12)]
+    min_val = (-2, -5, 0)
+    max_val = (7, 30, 15)
+    layer = Points(data)
+    extent = np.array((min_val, max_val))
+    check_layer_world_data_extent(layer, extent, (3, 1, 1), (10, 20, 5))
+
+
+def test_slice_data():
+    data = [
+        (10, 2, 4),
+        (10 + 2 * 1e-7, 4, 6),
+        (8, 1, 7),
+        (10.1, 7, 2),
+        (10 - 2 * 1e-7, 1, 6),
+    ]
+    layer = Points(data)
+    assert len(layer._slice_data((8, slice(None), slice(None)))[0]) == 1
+    assert len(layer._slice_data((10, slice(None), slice(None)))[0]) == 3
+    assert (
+        len(layer._slice_data((10 + 2 * 1e-12, slice(None), slice(None)))[0])
+        == 3
+    )
+    assert len(layer._slice_data((10.1, slice(None), slice(None)))[0]) == 1
+
+
+def test_scale_init():
+    layer = Points(None, scale=(1, 1, 1, 1))
+    assert layer.ndim == 4
+    layer1 = Points([], scale=(1, 1, 1, 1))
+    assert layer1.ndim == 4
+    layer2 = Points([])
+    assert layer2.ndim == 2
+
+    with pytest.raises(ValueError):
+        Points([[1, 1, 1]], scale=(1, 1, 1, 1))
+
+
+def test_update_none():
+    layer = Points([(1, 2, 3), (1, 3, 2)])
+    assert layer.ndim == 3
+    assert layer.data.size == 6
+    layer.data = None
+    assert layer.ndim == 3
+    assert layer.data.size == 0
+    layer.data = [(1, 2, 3), (1, 3, 2)]
+    assert layer.ndim == 3
+    assert layer.data.size == 6
+
+
+def test_prepare_properties():
+    layer = Points([(1, 2, 3), (1, 3, 2)])
+    properties, choices = layer._prepare_properties({"aa": [1, 2]})
+    assert list(properties.keys()) == ["aa"]
+    assert np.array_equal(properties["aa"], [1, 2])
+    assert list(choices.keys()) == ["aa"]
+    assert np.array_equal(choices["aa"], [1, 2])
+    assert layer._prepare_properties({}) == ({}, {})
+    assert layer._prepare_properties({}, {}) == ({}, {})
+    properties, choices = layer._prepare_properties({}, {"aa": [1, 2]})
+    assert list(properties.keys()) == ["aa"]
+    assert np.array_equal(properties["aa"], [None, None])
+    assert list(choices.keys()) == ["aa"]
+    assert np.array_equal(choices["aa"], [1, 2])
+    properties, choices = layer._prepare_properties(
+        {"aa": [1, 3]}, {"aa": [1, 2]}
+    )
+    assert list(properties.keys()) == ["aa"]
+    assert np.array_equal(properties["aa"], [1, 3])
+    assert list(choices.keys()) == ["aa"]
+    assert np.array_equal(choices["aa"], [1, 2, 3])
+    properties, choices = layer._prepare_properties(
+        {"aa": [1, 3]}, {"aa": [1, 2], "bb": [7, 6]}
+    )
+    assert list(properties.keys()) == ["aa"]
+    assert np.array_equal(properties["aa"], [1, 3])
+    assert list(choices.keys()) == ["aa"]
+    assert np.array_equal(choices["aa"], [1, 2, 3])
+    properties, choices = layer._prepare_properties(
+        {"aa": [1, 3]}, {"aa": [1, 2], "bb": [7, 6]}, save_choices=True
+    )
+    assert list(properties.keys()) == ["aa", "bb"]
+    assert np.array_equal(properties["aa"], [1, 3])
+    assert np.array_equal(properties["bb"], [None, None])
+    assert list(choices.keys()) == ["aa", "bb"]
+    assert np.array_equal(choices["aa"], [1, 2, 3])
+    assert np.array_equal(choices["bb"], [6, 7])
+
+    layer = Points([(1, 2, 3), (1, 3, 2), (1, 3, 3)])
+    properties, choices = layer._prepare_properties({"aa": [1, 2, 1]})
+    assert np.array_equal(properties["aa"], [1, 2, 1])
+    assert np.array_equal(choices["aa"], [1, 2])
