@@ -4,8 +4,8 @@ from itertools import cycle, islice
 import numpy as np
 import pandas as pd
 import pytest
-from vispy.color import get_colormap
 
+from napari._tests.utils import check_layer_world_data_extent
 from napari.layers import Shapes
 from napari.utils.colormaps.standardize_color import transform_color
 
@@ -31,7 +31,7 @@ def _make_cycled_properties(values, length):
 
 def test_empty_shapes():
     shp = Shapes()
-    assert shp.dims.ndim == 2
+    assert shp.ndim == 2
 
 
 properties_array = {'shape_type': _make_cycled_properties(['A', 'B'], 10)}
@@ -121,6 +121,32 @@ def test_adding_properties(attribute):
         layer.properties = properties_2
 
 
+def test_data_setter_with_properties():
+    """Test layer data on a layer with properties via the data setter"""
+    shape = (10, 4, 2)
+    np.random.seed(0)
+    data = 20 * np.random.random(shape)
+    properties = {'shape_type': _make_cycled_properties(['A', 'B'], shape[0])}
+    layer = Shapes(data, properties=properties)
+
+    # test setting to data with fewer shapes
+    n_new_shapes = 4
+    new_data = 20 * np.random.random((n_new_shapes, 4, 2))
+    layer.data = new_data
+    assert len(layer.properties['shape_type']) == n_new_shapes
+
+    # test setting to data with more shapes
+    n_new_shapes_2 = 6
+    new_data_2 = 20 * np.random.random((n_new_shapes_2, 4, 2))
+    layer.data = new_data_2
+    assert len(layer.properties['shape_type']) == n_new_shapes_2
+
+    # test setting to data with same shapes
+    new_data_3 = 20 * np.random.random((n_new_shapes_2, 4, 2))
+    layer.data = new_data_3
+    assert len(layer.properties['shape_type']) == n_new_shapes_2
+
+
 def test_properties_dataframe():
     """Test if properties can be provided as a DataFrame"""
     shape = (10, 4, 2)
@@ -133,6 +159,39 @@ def test_properties_dataframe():
     np.testing.assert_equal(layer.properties, properties)
 
 
+def test_empty_layer_with_text_properties():
+    """Test initializing an empty layer with text defined"""
+    default_properties = {'shape_type': np.array([1.5], dtype=float)}
+    text_kwargs = {'text': 'shape_type', 'color': 'red'}
+    layer = Shapes(
+        properties=default_properties,
+        text=text_kwargs,
+    )
+    assert layer.text.mode == 'property'
+    np.testing.assert_equal(layer.text.values, np.empty(0))
+    np.testing.assert_allclose(layer.text.color, [1, 0, 0, 1])
+
+    # add a shape and check that the appropriate text value was added
+    layer.add(np.random.random((1, 4, 2)))
+    np.testing.assert_equal(layer.text.values, ['1.5'])
+    np.testing.assert_allclose(layer.text.color, [1, 0, 0, 1])
+
+
+def test_empty_layer_with_text_formatted():
+    """Test initializing an empty layer with text defined"""
+    default_properties = {'shape_type': np.array([1.5], dtype=float)}
+    layer = Shapes(
+        properties=default_properties,
+        text='shape_type: {shape_type:.2f}',
+    )
+    assert layer.text.mode == 'formatted'
+    np.testing.assert_equal(layer.text.values, np.empty(0))
+
+    # add a shape and check that the appropriate text value was added
+    layer.add(np.random.random((1, 4, 2)))
+    np.testing.assert_equal(layer.text.values, ['shape_type: 1.50'])
+
+
 @pytest.mark.parametrize("properties", [properties_array, properties_list])
 def test_text_from_property_value(properties):
     """Test setting text from a property value"""
@@ -227,6 +286,52 @@ def test_refresh_text():
     np.testing.assert_equal(layer.text.values, new_properties['shape_type'])
 
 
+def test_nd_text():
+    """Test slicing of text coords with nD shapes"""
+    shapes_data = [
+        [[0, 10, 10, 10], [0, 10, 20, 20], [0, 10, 10, 20], [0, 10, 20, 10]],
+        [[1, 20, 30, 30], [1, 20, 50, 50], [1, 20, 50, 30], [1, 20, 30, 50]],
+    ]
+    properties = {'shape_type': ['A', 'B']}
+    text_kwargs = {'text': 'shape_type', 'anchor': 'center'}
+    layer = Shapes(shapes_data, properties=properties, text=text_kwargs)
+    assert layer.ndim == 4
+
+    layer._slice_dims(point=[0, 10, 0, 0], ndisplay=2)
+    np.testing.assert_equal(layer._indices_view, [0])
+    np.testing.assert_equal(layer._view_text_coords[0], [[15, 15]])
+
+    layer._slice_dims(point=[1, 0, 0, 0], ndisplay=3)
+    np.testing.assert_equal(layer._indices_view, [1])
+    np.testing.assert_equal(layer._view_text_coords[0], [[20, 40, 40]])
+
+
+@pytest.mark.parametrize("properties", [properties_array, properties_list])
+def test_data_setter_with_text(properties):
+    """Test layer data on a layer with text via the data setter"""
+    shape = (10, 4, 2)
+    np.random.seed(0)
+    data = 20 * np.random.random(shape)
+    layer = Shapes(data, properties=copy(properties), text='shape_type')
+
+    # test setting to data with fewer shapes
+    n_new_shapes = 4
+    new_data = 20 * np.random.random((n_new_shapes, 4, 2))
+    layer.data = new_data
+    assert len(layer.text.values) == n_new_shapes
+
+    # test setting to data with more shapes
+    n_new_shapes_2 = 6
+    new_data_2 = 20 * np.random.random((n_new_shapes_2, 4, 2))
+    layer.data = new_data_2
+    assert len(layer.text.values) == n_new_shapes_2
+
+    # test setting to data with same shapes
+    new_data_3 = 20 * np.random.random((n_new_shapes_2, 4, 2))
+    layer.data = new_data_3
+    assert len(layer.text.values) == n_new_shapes_2
+
+
 def test_rectangles():
     """Test instantiating Shapes layer with a random 2D rectangles."""
     # Test a single four corner rectangle
@@ -268,6 +373,38 @@ def test_rectangles():
     assert np.all([s == 'rectangle' for s in layer.shape_type])
 
 
+def test_rectangles_with_shape_type():
+    """Test instantiating rectangles with shape_type in data"""
+    # Test (rectangle, shape_type) tuple
+    shape = (1, 4, 2)
+    np.random.seed(0)
+    vertices = 20 * np.random.random(shape)
+    data = (vertices, "rectangle")
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all(layer.data[0] == data[0])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'rectangle' for s in layer.shape_type])
+
+    # Test (list of rectangles, shape_type) tuple
+    shape = (10, 4, 2)
+    vertices = 20 * np.random.random(shape)
+    data = (vertices, "rectangle")
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, vertices)])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'rectangle' for s in layer.shape_type])
+
+    # Test list of (rectangle, shape_type) tuples
+    data = [(vertices[i], "rectangle") for i in range(shape[0])]
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, vertices)])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'rectangle' for s in layer.shape_type])
+
+
 def test_rectangles_roundtrip():
     """Test a full roundtrip with rectangles data."""
     shape = (10, 4, 2)
@@ -372,6 +509,74 @@ def test_ellipses():
     assert np.all([s == 'ellipse' for s in layer.shape_type])
 
 
+def test_ellipses_with_shape_type():
+    """Test instantiating ellipses with shape_type in data"""
+    # Test single four corner (vertices, shape_type) tuple
+    shape = (1, 4, 2)
+    np.random.seed(0)
+    vertices = 20 * np.random.random(shape)
+    data = (vertices, "ellipse")
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all(layer.data[0] == data[0])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+    # Test multiple four corner (list of vertices, shape_type) tuple
+    shape = (10, 4, 2)
+    np.random.seed(0)
+    vertices = 20 * np.random.random(shape)
+    data = (vertices, "ellipse")
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, vertices)])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+    # Test list of four corner (vertices, shape_type) tuples
+    shape = (10, 4, 2)
+    np.random.seed(0)
+    vertices = 20 * np.random.random(shape)
+    data = [(vertices[i], "ellipse") for i in range(shape[0])]
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, vertices)])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+    # Test single (center-radii, shape_type) ellipse
+    shape = (1, 2, 2)
+    np.random.seed(0)
+    data = (20 * np.random.random(shape), "ellipse")
+    layer = Shapes(data)
+    assert layer.nshapes == 1
+    assert len(layer.data[0]) == 4
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+    # Test (list of center-radii, shape_type) tuple
+    shape = (10, 2, 2)
+    np.random.seed(0)
+    center_radii = 20 * np.random.random(shape)
+    data = (center_radii, "ellipse")
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([len(ld) == 4 for ld in layer.data])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+    # Test list of (center-radii, shape_type) tuples
+    shape = (10, 2, 2)
+    np.random.seed(0)
+    center_radii = 20 * np.random.random(shape)
+    data = [(center_radii[i], "ellipse") for i in range(shape[0])]
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([len(ld) == 4 for ld in layer.data])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+
 def test_4D_ellispse():
     """Test instantiating Shapes layer with 4D planar ellipse."""
     # Test a single 4D ellipse
@@ -424,6 +629,42 @@ def test_lines():
     assert np.all([s == 'line' for s in layer.shape_type])
 
 
+def test_lines_with_shape_type():
+    """Test instantiating lines with shape_type"""
+    # Test (single line, shape_type) tuple
+    shape = (1, 2, 2)
+    np.random.seed(0)
+    end_points = 20 * np.random.random(shape)
+    data = (end_points, 'line')
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all(layer.data[0] == end_points[0])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'line' for s in layer.shape_type])
+
+    # Test (multiple lines, shape_type) tuple
+    shape = (10, 2, 2)
+    np.random.seed(0)
+    end_points = 20 * np.random.random(shape)
+    data = (end_points, "line")
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, end_points)])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'line' for s in layer.shape_type])
+
+    # Test list of (line, shape_type) tuples
+    shape = (10, 2, 2)
+    np.random.seed(0)
+    end_points = 20 * np.random.random(shape)
+    data = [(end_points[i], "line") for i in range(shape[0])]
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, end_points)])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'line' for s in layer.shape_type])
+
+
 def test_lines_roundtrip():
     """Test a full roundtrip with line data."""
     shape = (10, 2, 2)
@@ -457,6 +698,39 @@ def test_paths():
     assert np.all([s == 'path' for s in layer.shape_type])
 
 
+def test_paths_with_shape_type():
+    """Test instantiating paths with shape_type in data"""
+    # Test (single path, shape_type) tuple
+    shape = (1, 6, 2)
+    np.random.seed(0)
+    path_points = 20 * np.random.random(shape)
+    data = (path_points, "path")
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all(layer.data[0] == path_points[0])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'path' for s in layer.shape_type])
+
+    # Test (list of paths, shape_type) tuple
+    path_points = [
+        20 * np.random.random((np.random.randint(2, 12), 2)) for i in range(10)
+    ]
+    data = (path_points, "path")
+    layer = Shapes(data)
+    assert layer.nshapes == len(path_points)
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, path_points)])
+    assert layer.ndim == 2
+    assert np.all([s == 'path' for s in layer.shape_type])
+
+    # Test list of  (path, shape_type) tuples
+    data = [(path_points[i], "path") for i in range(len(path_points))]
+    layer = Shapes(data)
+    assert layer.nshapes == len(data)
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, path_points)])
+    assert layer.ndim == 2
+    assert np.all([s == 'path' for s in layer.shape_type])
+
+
 def test_paths_roundtrip():
     """Test a full roundtrip with path data."""
     np.random.seed(0)
@@ -493,6 +767,40 @@ def test_polygons():
     assert np.all([s == 'polygon' for s in layer.shape_type])
 
 
+def test_polygons_with_shape_type():
+    """Test 2D polygons with shape_type in data"""
+
+    # Test single (polygon, shape_type) tuple
+    shape = (1, 6, 2)
+    np.random.seed(0)
+    vertices = 20 * np.random.random(shape)
+    data = (vertices, 'polygon')
+    layer = Shapes(data)
+    assert layer.nshapes == shape[0]
+    assert np.all(layer.data[0] == vertices[0])
+    assert layer.ndim == shape[2]
+    assert np.all([s == 'polygon' for s in layer.shape_type])
+
+    # Test (list of polygons, shape_type) tuple
+    polygons = [
+        20 * np.random.random((np.random.randint(2, 12), 2)) for i in range(10)
+    ]
+    data = (polygons, 'polygon')
+    layer = Shapes(data)
+    assert layer.nshapes == len(polygons)
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, polygons)])
+    assert layer.ndim == 2
+    assert np.all([s == 'polygon' for s in layer.shape_type])
+
+    # Test list of (polygon, shape_type) tuples
+    data = [(polygons[i], 'polygon') for i in range(len(polygons))]
+    layer = Shapes(data)
+    assert layer.nshapes == len(polygons)
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, polygons)])
+    assert layer.ndim == 2
+    assert np.all([s == 'polygon' for s in layer.shape_type])
+
+
 def test_polygon_roundtrip():
     """Test a full roundtrip with polygon data."""
     np.random.seed(0)
@@ -510,13 +818,15 @@ def test_mixed_shapes():
     """Test instantiating Shapes layer with a mix of random 2D shapes."""
     # Test multiple polygons with different numbers of points
     np.random.seed(0)
-    data = [
+    shape_vertices = [
         20 * np.random.random((np.random.randint(2, 12), 2)) for i in range(5)
     ] + list(np.random.random((5, 4, 2)))
     shape_type = ['polygon'] * 5 + ['rectangle'] * 3 + ['ellipse'] * 2
-    layer = Shapes(data, shape_type=shape_type)
-    assert layer.nshapes == len(data)
-    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, data)])
+    layer = Shapes(shape_vertices, shape_type=shape_type)
+    assert layer.nshapes == len(shape_vertices)
+    assert np.all(
+        [np.all(ld == d) for ld, d in zip(layer.data, shape_vertices)]
+    )
     assert layer.ndim == 2
     assert np.all([s == so for s, so in zip(layer.shape_type, shape_type)])
 
@@ -530,21 +840,99 @@ def test_mixed_shapes():
     )
 
 
+def test_mixed_shapes_with_shape_type():
+    """Test adding mixed shapes with shape_type in data"""
+    np.random.seed(0)
+    shape_vertices = [
+        20 * np.random.random((np.random.randint(2, 12), 2)) for i in range(5)
+    ] + list(np.random.random((5, 4, 2)))
+    shape_type = ['polygon'] * 5 + ['rectangle'] * 3 + ['ellipse'] * 2
+
+    # Test multiple (shape, shape_type) tuples
+    data = list(zip(shape_vertices, shape_type))
+    layer = Shapes(data)
+    assert layer.nshapes == len(shape_vertices)
+    assert np.all(
+        [np.all(ld == d) for ld, d in zip(layer.data, shape_vertices)]
+    )
+    assert layer.ndim == 2
+    assert np.all([s == so for s, so in zip(layer.shape_type, shape_type)])
+
+
+def test_data_shape_type_overwrites_meta():
+    """Test shape type passed through data property overwrites metadata shape type"""
+    shape = (10, 4, 2)
+    np.random.seed(0)
+    vertices = 20 * np.random.random(shape)
+    data = (vertices, "ellipse")
+    layer = Shapes(data, shape_type='rectangle')
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+    data = [(vertices[i], "ellipse") for i in range(shape[0])]
+    layer = Shapes(data, shape_type='rectangle')
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+
 def test_changing_shapes():
     """Test changing Shapes data."""
     shape_a = (10, 4, 2)
     shape_b = (20, 4, 2)
     np.random.seed(0)
-    data_a = 20 * np.random.random(shape_a)
-    data_b = 20 * np.random.random(shape_b)
-    layer = Shapes(data_a)
+    vertices_a = 20 * np.random.random(shape_a)
+    vertices_b = 20 * np.random.random(shape_b)
+    layer = Shapes(vertices_a)
     assert layer.nshapes == shape_a[0]
-    layer.data = data_b
+    layer.data = vertices_b
     assert layer.nshapes == shape_b[0]
-    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, data_b)])
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, vertices_b)])
     assert layer.ndim == shape_b[2]
     assert np.all([s == 'rectangle' for s in layer.shape_type])
 
+    # setting data with shape type
+    data_a = (vertices_a, "ellipse")
+    layer.data = data_a
+    assert layer.nshapes == shape_a[0]
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, vertices_a)])
+    assert layer.ndim == shape_a[2]
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
+    # setting data with fewer shapes
+    smaller_data = vertices_a[:5]
+    current_edge_color = layer._data_view.edge_color
+    current_edge_width = layer._data_view.edge_widths
+    current_face_color = layer._data_view.face_color
+    current_z = layer._data_view.z_indices
+
+    layer.data = smaller_data
+    assert layer.nshapes == smaller_data.shape[0]
+    assert np.allclose(layer._data_view.edge_color, current_edge_color[:5])
+    assert np.allclose(layer._data_view.face_color, current_face_color[:5])
+    assert np.allclose(layer._data_view.edge_widths, current_edge_width[:5])
+    assert np.allclose(layer._data_view.z_indices, current_z[:5])
+
+    # setting data with added shapes
+    current_edge_color = layer._data_view.edge_color
+    current_edge_width = layer._data_view.edge_widths
+    current_face_color = layer._data_view.face_color
+    current_z = layer._data_view.z_indices
+
+    bigger_data = vertices_b
+    layer.data = bigger_data
+    assert layer.nshapes == bigger_data.shape[0]
+    assert np.allclose(layer._data_view.edge_color[:5], current_edge_color)
+    assert np.allclose(layer._data_view.face_color[:5], current_face_color)
+    assert np.allclose(layer._data_view.edge_widths[:5], current_edge_width)
+    assert np.allclose(layer._data_view.z_indices[:5], current_z)
+
+
+def test_changing_shape_type():
+    """Test changing shape type"""
+    np.random.seed(0)
+    rectangles = 20 * np.random.random((10, 4, 2))
+    layer = Shapes(rectangles, shape_type='rectangle')
+    layer.shape_type = "ellipse"
+    assert np.all([s == 'ellipse' for s in layer.shape_type])
+
 
 def test_adding_shapes():
     """Test adding shapes."""
@@ -565,6 +953,18 @@ def test_adding_shapes():
     assert layer.ndim == 2
     assert np.all([s == so for s, so in zip(layer.shape_type, all_shape_type)])
 
+    # test adding data with shape_type
+    new_vertices = np.random.random((5, 4, 2))
+    new_shape_type2 = ['ellipse'] * 3 + ['rectangle'] * 2
+    new_data2 = list(zip(new_vertices, new_shape_type2))
+    layer.add(new_data2)
+    all_vertices = all_data + list(new_vertices)
+    all_shape_type = all_shape_type + new_shape_type2
+    assert layer.nshapes == len(all_vertices)
+    assert np.all([np.all(ld == d) for ld, d in zip(layer.data, all_vertices)])
+    assert layer.ndim == 2
+    assert np.all([s == so for s, so in zip(layer.shape_type, all_shape_type)])
+
 
 def test_adding_shapes_to_empty():
     """Test adding shapes to empty."""
@@ -600,6 +1000,28 @@ def test_selecting_shapes():
     assert layer.selected_data == set()
 
 
+def test_removing_all_shapes_empty_list():
+    """Test removing all shapes with an empty list."""
+    data = 20 * np.random.random((10, 4, 2))
+    np.random.seed(0)
+    layer = Shapes(data)
+    assert layer.nshapes == 10
+
+    layer.data = []
+    assert layer.nshapes == 0
+
+
+def test_removing_all_shapes_empty_array():
+    """Test removing all shapes with an empty list."""
+    data = 20 * np.random.random((10, 4, 2))
+    np.random.seed(0)
+    layer = Shapes(data)
+    assert layer.nshapes == 10
+
+    layer.data = np.empty((0, 2))
+    assert layer.nshapes == 0
+
+
 def test_removing_selected_shapes():
     """Test removing selected shapes."""
     np.random.seed(0)
@@ -757,6 +1179,7 @@ def test_blending():
     assert layer.blending == 'opaque'
 
 
+@pytest.mark.filterwarnings("ignore:elementwise comparison fail:FutureWarning")
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 def test_switch_color_mode(attribute):
     """Test switching between color modes"""
@@ -807,7 +1230,7 @@ def test_switch_color_mode(attribute):
     setattr(layer, f'{attribute}_color_mode', 'cycle')
     setattr(layer, f'{attribute}_color', 'shape_type')
     color = getattr(layer, f'{attribute}_color')
-    layer_color = transform_color(color_cycle * int((shape[0] / 2)))
+    layer_color = transform_color(color_cycle * int(shape[0] / 2))
     np.testing.assert_allclose(color, layer_color)
 
     # switch back to direct, edge_colors shouldn't change
@@ -848,7 +1271,6 @@ def test_color_direct(attribute: str):
     color_array[list(selected_data)] = colorarray_green
     layer_color = getattr(layer, f'{attribute}_color')
     np.testing.assert_allclose(color_array, layer_color)
-
     # Add new shape and test its color
     new_shape = np.random.random((1, 4, 2))
     layer.selected_data = set()
@@ -867,7 +1289,8 @@ def test_color_direct(attribute: str):
     layer_color = getattr(layer, f'{attribute}_color')
     assert len(layer_color) == shape[0] - 1
     np.testing.assert_allclose(
-        layer_color, np.vstack((color_array[1], color_array[3:])),
+        layer_color,
+        np.vstack((color_array[1], color_array[3:])),
     )
 
     # set the color directly
@@ -877,6 +1300,19 @@ def test_color_direct(attribute: str):
     np.testing.assert_allclose(color_array, layer_color)
 
 
+@pytest.mark.parametrize("attribute", ['edge', 'face'])
+def test_single_shape_properties(attribute):
+    """Test creating single shape with properties"""
+    shape = (4, 2)
+    np.random.seed(0)
+    data = 20 * np.random.random(shape)
+    layer_kwargs = {f'{attribute}_color': 'red'}
+    layer = Shapes(data, **layer_kwargs)
+    layer_color = getattr(layer, f'{attribute}_color')
+    assert len(layer_color) == 1
+    np.testing.assert_allclose([1, 0, 0, 1], layer_color[0])
+
+
 color_cycle_str = ['red', 'blue']
 color_cycle_rgb = [[1, 0, 0], [0, 0, 1]]
 color_cycle_rgba = [[1, 0, 0, 1], [0, 0, 1, 1]]
@@ -884,7 +1320,8 @@ color_cycle_rgba = [[1, 0, 0, 1], [0, 0, 1, 1]]
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 @pytest.mark.parametrize(
-    "color_cycle", [color_cycle_str, color_cycle_rgb, color_cycle_rgba],
+    "color_cycle",
+    [color_cycle_str, color_cycle_rgb, color_cycle_rgba],
 )
 def test_color_cycle(attribute, color_cycle):
     """Test setting edge/face color with a color cycle list"""
@@ -914,7 +1351,8 @@ def test_color_cycle(attribute, color_cycle):
     layer_color = getattr(layer, f'{attribute}_color')
     assert len(layer_color) == shape[0] + 1
     np.testing.assert_allclose(
-        layer_color, np.vstack((color_array, transform_color('red'))),
+        layer_color,
+        np.vstack((color_array, transform_color('red'))),
     )
 
     # Check removing data adjusts colors correctly
@@ -949,7 +1387,7 @@ def test_color_cycle(attribute, color_cycle):
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 def test_add_color_cycle_to_empty_layer(attribute):
-    """ Test adding a shape to an empty layer when edge/face color is a color cycle
+    """Test adding a shape to an empty layer when edge/face color is a color cycle
 
     See: https://github.com/napari/napari/pull/1069
     """
@@ -992,7 +1430,7 @@ def test_add_color_cycle_to_empty_layer(attribute):
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
 def test_adding_value_color_cycle(attribute):
-    """ Test that adding values to properties used to set a color cycle
+    """Test that adding values to properties used to set a color cycle
     and then calling Shapes.refresh_colors() performs the update and adds the
     new value to the face/edge_color_cycle_map.
     """
@@ -1036,7 +1474,7 @@ def test_color_colormap(attribute):
     assert layer.properties == properties
     color_mode = getattr(layer, f'{attribute}_color_mode')
     assert color_mode == 'colormap'
-    color_array = transform_color(['black', 'white'] * int((shape[0] / 2)))
+    color_array = transform_color(['black', 'white'] * int(shape[0] / 2))
     attribute_color = getattr(layer, f'{attribute}_color')
     assert np.all(attribute_color == color_array)
 
@@ -1052,7 +1490,8 @@ def test_color_colormap(attribute):
     attribute_color = getattr(layer, f'{attribute}_color')
     assert len(attribute_color) == shape[0] + 1
     np.testing.assert_allclose(
-        attribute_color, np.vstack((color_array, transform_color('black'))),
+        attribute_color,
+        np.vstack((color_array, transform_color('black'))),
     )
 
     # Check removing data adjusts colors correctly
@@ -1064,7 +1503,11 @@ def test_color_colormap(attribute):
     np.testing.assert_allclose(
         attribute_color,
         np.vstack(
-            (color_array[1], color_array[3:], transform_color('black'),)
+            (
+                color_array[1],
+                color_array[3:],
+                transform_color('black'),
+            )
         ),
     )
 
@@ -1078,7 +1521,7 @@ def test_color_colormap(attribute):
     new_colormap = 'viridis'
     setattr(layer, f'{attribute}_colormap', new_colormap)
     attribute_colormap = getattr(layer, f'{attribute}_colormap')
-    assert attribute_colormap[1] == get_colormap(new_colormap)
+    assert attribute_colormap.name == new_colormap
 
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
@@ -1103,7 +1546,8 @@ def test_colormap_with_categorical_properties(attribute):
     layer = Shapes(data, properties=properties)
 
     with pytest.raises(TypeError):
-        setattr(layer, f'{attribute}_color_mode', 'colormap')
+        with pytest.warns(UserWarning):
+            setattr(layer, f'{attribute}_color_mode', 'colormap')
 
 
 @pytest.mark.parametrize("attribute", ['edge', 'face'])
@@ -1118,9 +1562,9 @@ def test_add_colormap(attribute):
     args = {color_kwarg: 'shape_type', colormap_kwarg: 'viridis'}
     layer = Shapes(data, properties=annotations, **args)
 
-    setattr(layer, f'{attribute}_colormap', get_colormap('gray'))
+    setattr(layer, f'{attribute}_colormap', 'gray')
     layer_colormap = getattr(layer, f'{attribute}_colormap')
-    assert 'unnamed colormap' in layer_colormap[0]
+    assert layer_colormap.name == 'gray'
 
 
 def test_edge_width():
@@ -1174,6 +1618,20 @@ def test_edge_width():
     assert len(layer.edge_width) == shape[0] - 1
     assert layer.edge_width == [width_list[1]] + width_list[3:] + [4]
 
+    # Test setting edge width with number
+    layer.edge_width = 4
+    assert all([width == 4 for width in layer.edge_width])
+
+    # Test setting edge width with list
+    new_widths = [2] * 5 + [3] * 4
+    layer.edge_width = new_widths
+    assert layer.edge_width == new_widths
+
+    # Test setting with incorrect size list throws error
+    new_widths = [2, 3]
+    with pytest.raises(ValueError):
+        layer.edge_width = new_widths
+
 
 def test_z_index():
     """Test setting z-index during instantiation."""
@@ -1205,6 +1663,20 @@ def test_z_index():
     assert len(layer.z_index) == shape[0] - 1
     assert layer.z_index == [z_index_list[1]] + z_index_list[3:] + [4]
 
+    # Test setting index with number
+    layer.z_index = 4
+    assert all([idx == 4 for idx in layer.z_index])
+
+    # Test setting index with list
+    new_z_indices = [2] * 5 + [3] * 4
+    layer.z_index = new_z_indices
+    assert layer.z_index == new_z_indices
+
+    # Test setting with incorrect size list throws error
+    new_z_indices = [2, 3]
+    with pytest.raises(ValueError):
+        layer.z_index = new_z_indices
+
 
 def test_move_to_front():
     """Test moving shapes to front."""
@@ -1304,17 +1776,16 @@ def test_value():
     data = 20 * np.random.random(shape)
     data[-1, :] = [[0, 0], [0, 10], [10, 0], [10, 10]]
     layer = Shapes(data)
-    value = layer.get_value()
-    assert layer.coordinates == (0, 0)
+    value = layer.get_value((0,) * 2)
     assert value == (9, None)
 
     layer.mode = 'select'
     layer.selected_data = {9}
-    value = layer.get_value()
+    value = layer.get_value((0,) * 2)
     assert value == (9, 7)
 
     layer = Shapes(data + 5)
-    value = layer.get_value()
+    value = layer.get_value((0,) * 2)
     assert value == (None, None)
 
 
@@ -1324,7 +1795,7 @@ def test_message():
     np.random.seed(0)
     data = 20 * np.random.random(shape)
     layer = Shapes(data)
-    msg = layer.get_message()
+    msg = layer.get_status((0,) * 2)
     assert type(msg) == str
 
 
@@ -1418,3 +1889,13 @@ def test_add_shapes_consistent_properties():
     assert len(layer.properties['index']) == 4
     assert layer.properties['index'][2] == 2
     assert layer.properties['index'][3] == 2
+
+
+def test_world_data_extent():
+    """Test extent after applying transforms."""
+    data = [(7, -5, 0), (-2, 0, 15), (4, 30, 12)]
+    layer = Shapes([data, np.add(data, [2, -3, 0])], shape_type='polygon')
+    min_val = (-2, -8, 0)
+    max_val = (9, 30, 15)
+    extent = np.array((min_val, max_val))
+    check_layer_world_data_extent(layer, extent, (3, 1, 1), (10, 20, 5))
