@@ -1,11 +1,13 @@
+import collections
+
 import numpy as np
 import pytest
-import collections
+
 from napari.layers import Shapes
 from napari.utils.interactions import (
     ReadOnlyWrapper,
-    mouse_press_callbacks,
     mouse_move_callbacks,
+    mouse_press_callbacks,
     mouse_release_callbacks,
 )
 
@@ -21,7 +23,7 @@ def Event():
         NamedTuple object with fields "type", "is_dragging", and "modifiers".
     """
     return collections.namedtuple(
-        'Event', field_names=['type', 'is_dragging', 'modifiers']
+        'Event', field_names=['type', 'is_dragging', 'modifiers', 'position']
     )
 
 
@@ -58,13 +60,23 @@ def test_not_adding_or_selecting_shape(create_known_shapes_layer, Event):
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=(0, 0),
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=(0, 0),
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -80,25 +92,38 @@ def test_add_simple_shape(shape_type, create_known_shapes_layer, Event):
 
     # Add shape at location where non exists
     layer.mode = 'add_' + shape_type
-    layer.position = known_non_shape
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     known_non_shape_end = [40, 60]
-    layer.position = known_non_shape_end
     # Simulate drag end
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=known_non_shape_end,
+        )
     )
     mouse_move_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape_end,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -118,21 +143,34 @@ def test_add_complex_shape(shape_type, create_known_shapes_layer, Event):
     desired_shape = [[20, 30], [10, 50], [60, 40], [80, 20]]
     # Add shape at location where non exists
     layer.mode = 'add_' + shape_type
-    layer.position = desired_shape[0]
 
     for coord in desired_shape:
-        layer.position = coord
         # Simulate move, click, and release
         event = ReadOnlyWrapper(
-            Event(type='mouse_move', is_dragging=False, modifiers=[])
+            Event(
+                type='mouse_move',
+                is_dragging=False,
+                modifiers=[],
+                position=coord,
+            )
         )
         mouse_move_callbacks(layer, event)
         event = ReadOnlyWrapper(
-            Event(type='mouse_press', is_dragging=False, modifiers=[])
+            Event(
+                type='mouse_press',
+                is_dragging=False,
+                modifiers=[],
+                position=coord,
+            )
         )
         mouse_press_callbacks(layer, event)
         event = ReadOnlyWrapper(
-            Event(type='mouse_release', is_dragging=False, modifiers=[])
+            Event(
+                type='mouse_release',
+                is_dragging=False,
+                modifiers=[],
+                position=coord,
+            )
         )
         mouse_release_callbacks(layer, event)
 
@@ -152,17 +190,26 @@ def test_vertex_insert(create_known_shapes_layer, Event):
     n_coord = len(layer.data[0])
     layer.mode = 'vertex_insert'
     layer.selected_data = {0}
-    layer.position = known_non_shape
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate drag end
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_move_callbacks(layer, event)
 
@@ -181,17 +228,27 @@ def test_vertex_remove(create_known_shapes_layer, Event):
     n_coord = len(layer.data[0])
     layer.mode = 'vertex_remove'
     layer.selected_data = {0}
-    layer.position = tuple(layer.data[0][0])
+    position = tuple(layer.data[0][0])
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate drag end
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_move_callbacks(layer, event)
 
@@ -206,17 +263,27 @@ def test_select_shape(mode, create_known_shapes_layer, Event):
     layer, n_shapes, _ = create_known_shapes_layer
 
     layer.mode = mode
-    layer.position = tuple(layer.data[0][0])
+    position = tuple(layer.data[0][0])
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -230,44 +297,84 @@ def test_drag_shape(create_known_shapes_layer, Event):
     layer, n_shapes, _ = create_known_shapes_layer
 
     layer.mode = 'select'
+    # Zoom in so as to not select any vertices
+    layer.scale_factor = 0.01
     orig_data = layer.data[0].copy()
     assert len(layer.selected_data) == 0
 
-    layer.position = tuple(np.mean(layer.data[0], axis=0))
+    position = tuple(np.mean(layer.data[0], axis=0))
+
+    # Check shape under cursor
+    value = layer.get_value(position, world=True)
+    assert value == (0, None)
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_press_callbacks(layer, event)
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_release_callbacks(layer, event)
 
     assert len(layer.selected_data) == 1
     assert layer.selected_data == {0}
 
+    # Check shape but not vertex under cursor
+    value = layer.get_value(event.position, world=True)
+    assert value == (0, None)
+
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=True,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_press_callbacks(layer, event)
     # start drag event
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_move_callbacks(layer, event)
-    layer.position = tuple(np.add(layer.position, [10, 5]))
+    position = tuple(np.add(position, [10, 5]))
     # Simulate move, click, and release
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_move_callbacks(layer, event)
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=True,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -283,24 +390,39 @@ def test_drag_vertex(create_known_shapes_layer, Event):
 
     layer.mode = 'direct'
     layer.selected_data = {0}
-    layer.position = tuple(layer.data[0][0])
+    position = tuple(layer.data[0][0])
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_press_callbacks(layer, event)
 
-    layer.position = [0, 0]
+    position = [0, 0]
     # Simulate move, click, and release
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_move_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=True,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -330,17 +452,27 @@ def test_after_in_add_mode_shape(mode, create_known_shapes_layer, Event):
 
     layer.mode = mode
     layer.mode = 'pan_zoom'
-    layer.position = tuple(layer.data[0][0])
+    position = tuple(layer.data[0][0])
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -355,18 +487,28 @@ def test_unselect_select_shape(mode, create_known_shapes_layer, Event):
     layer, n_shapes, _ = create_known_shapes_layer
 
     layer.mode = mode
-    layer.position = tuple(layer.data[0][0])
+    position = tuple(layer.data[0][0])
     layer.selected_data = {1}
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=position,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -381,17 +523,26 @@ def test_not_selecting_shape(mode, create_known_shapes_layer, Event):
     layer, n_shapes, known_non_shape = create_known_shapes_layer
 
     layer.mode = mode
-    layer.position = known_non_shape
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -405,19 +556,28 @@ def test_unselecting_shapes(mode, create_known_shapes_layer, Event):
     layer, n_shapes, known_non_shape = create_known_shapes_layer
 
     layer.mode = mode
-    layer.position = known_non_shape
     layer.selected_data = {0, 1}
     assert len(layer.selected_data) == 2
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -431,30 +591,45 @@ def test_selecting_shapes_with_drag(mode, create_known_shapes_layer, Event):
     layer, n_shapes, known_non_shape = create_known_shapes_layer
 
     layer.mode = mode
-    layer.position = known_non_shape
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate drag start
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_move_callbacks(layer, event)
 
-    layer.position = [0, 0]
     # Simulate drag end
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move', is_dragging=True, modifiers=[], position=(0, 0)
+        )
     )
     mouse_move_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=True,
+            modifiers=[],
+            position=(0, 0),
+        )
     )
     mouse_release_callbacks(layer, event)
 
@@ -468,30 +643,48 @@ def test_selecting_no_shapes_with_drag(mode, create_known_shapes_layer, Event):
     layer, n_shapes, known_non_shape = create_known_shapes_layer
 
     layer.mode = mode
-    layer.position = known_non_shape
 
     # Simulate click
     event = ReadOnlyWrapper(
-        Event(type='mouse_press', is_dragging=False, modifiers=[])
+        Event(
+            type='mouse_press',
+            is_dragging=False,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_press_callbacks(layer, event)
 
     # Simulate drag start
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=known_non_shape,
+        )
     )
     mouse_move_callbacks(layer, event)
 
-    layer.position = [50, 60]
     # Simulate drag end
     event = ReadOnlyWrapper(
-        Event(type='mouse_move', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_move',
+            is_dragging=True,
+            modifiers=[],
+            position=(50, 60),
+        )
     )
     mouse_move_callbacks(layer, event)
 
     # Simulate release
     event = ReadOnlyWrapper(
-        Event(type='mouse_release', is_dragging=True, modifiers=[])
+        Event(
+            type='mouse_release',
+            is_dragging=True,
+            modifiers=[],
+            position=(50, 60),
+        )
     )
     mouse_release_callbacks(layer, event)
 
